'''
Author: your name
Date: 2021-06-29 19:24:32
LastEditTime: 2021-08-07 18:05:28
LastEditors: Please set LastEditors
Description: In User Settings Edit
FilePath: \evm-store\backend\controller\monitor.py
'''
import jwt
import json
import time
import logging
from copy import deepcopy
from app import config
from model.monitor import session, System, Lvgl, Evm, Image, Device, Request, User

logger = logging.getLogger(__name__)

class SystemResource(object):
    def get(self):
        result = session.query(System).all()
        print(result)
        return result

    def post(self, params):
        data = dict()
        for k in params:
            if hasattr(System, k):
                data[k] = params[k]
        result = System(**data)
        session.add(result)
        return session.commit()
        
    def put(self):
        pass

    def delete(self):
        pass

class LvglResource(object):
    def get(self):
        result = session.query(Lvgl).all()
        print(result)
        return result

    def post(self, params):
        data = dict()
        for k in params:
            if hasattr(Lvgl, k):
                data[k] = params[k]
        result = Lvgl(**data)
        session.add(result)
        return session.commit()
        
    def put(self):
        pass

    def delete(self):
        pass

class EvmResource(object):
    def get(self):
        result = session.query(Evm).all()
        print(result)
        return result

    def post(self, params):
        data = dict()
        for k in params:
            if hasattr(Evm, k):
                data[k] = params[k]
        result = Evm(**data)
        session.add(result)
        return session.commit()

    def put(self):
        pass

    def delete(self):
        pass

class ImageResource(object):
    def get(self):
        result = session.query(Image).all()
        print(result)
        return result

    def post(self, params):
        data = dict()
        for k in params:
            if hasattr(Image, k):
                data[k] = params[k]
        result = Image(**data)
        session.add(result)
        return session.commit()

    def post_array(self, array, watch):
        t = []
        for a in array:
            a.update({ "watch": watch })
            data = dict()
            for k in a:
                if hasattr(Image, k):
                    data[k] = a[k]
            t.append(Image(**data))
        session.add_all(t)
        return session.commit()
        
    def put(self):
        pass

    def delete(self):
        pass

systemResource = SystemResource()
lvglResource = LvglResource()
evmResource = EvmResource()
imageResource = ImageResource()

def get_watch_list():
    result = session.query(Device).all()
    tmp = []
    for item in result:
        tmp.append({
            'id': item.id,
            'imei': item.imei
        })
    return tmp

def evm_data(watch, start, end):
    filters = [Evm.watch==watch]
    if start:
        filters.append(Evm.timestamp >= start)
    if end:
        filters.append(Evm.timestamp <= end)
    result = session.query(Evm).filter(*filters).order_by(Evm.timestamp).all()
    temp = []
    for item in result:
        t = item.to_dict()
        if t.get("timestamp"):
            t.update({ 'timestamp': t.get("timestamp").strftime("%Y-%m-%d %H:%M:%S") })
        temp.append(t)

    return temp

def lvgl_data(watch, start, end):
    filters = [Lvgl.watch==watch]
    if start:
        filters.append(Lvgl.timestamp>=start)
    if end:
        filters.append(Lvgl.timestamp<=end)
    result = session.query(Lvgl).filter(*filters).order_by(Lvgl.timestamp).all()
    temp = []
    for item in result:
        t = item.to_dict()
        if t.get("timestamp"):
            t.update({ 'timestamp': t.get("timestamp").strftime("%Y-%m-%d %H:%M:%S") })
        temp.append(t)

    return temp

def image_data(watch, start, end):
    filters = [Image.watch==watch]
    if start:
        filters.append(Image.timestamp>=start)
    if end:
        filters.append(Image.timestamp<=end)
    result = session.query(Image).filter(*filters).order_by(Image.timestamp).all()
    temp = []
    for item in result:
        t = item.to_dict()
        if t.get("timestamp"):
            t.update({ 'timestamp': t.get("timestamp").strftime("%Y-%m-%d %H:%M:%S") })
        temp.append(t)

    return temp

def get_monitor_list(watch, category, start, end):
    # 判断watch是否存在
    w = session.query(Device).filter(Device.id==watch).first()
    if not w:
        return []

    if category == "system":
        return []
    elif category == "image":
        return image_data(watch, start, end)
    elif category == "lvgl":
        return lvgl_data(watch, start, end)
    elif category == "evm":
        return evm_data(watch, start, end)
    else:
        return {
            'evm': evm_data(watch, start, end),
            'lvgl': lvgl_data(watch, start, end),
            'image': image_data(watch, start, end)
        }

class MonitorManager(object):
    def __init__(self):
        pass

    def get(self, data):
        watch = data.get("watch")
        category = data.get("category")
        start = data.get("start")
        end = data.get("end")

        # 判断watch是否存在
        w = session.query(Device).filter(Device.id==watch).first()
        if not w:
            return []

        if category == "system":
            return []
        elif category == "image":
            return image_data(watch, start, end)
        elif category == "lvgl":
            return lvgl_data(watch, start, end)
        elif category == "evm":
            return evm_data(watch, start, end)
        else:
            return {
                'evm': evm_data(watch, start, end),
                'lvgl': lvgl_data(watch, start, end),
                'image': image_data(watch, start, end)
            }

    def post(self, msg):
        # 先判断手表imei是否存在，不存在则先注册手表IMEI
        watch_id = -1
        if msg.get("imei"):
            result = session.query(Device).filter_by(imei=msg.get("imei")).first()
            if result:
                watch_id = result.id
            else:
                logger.info("设备不存在")
                return None

        if msg.get("request"):
            obj = deepcopy(msg.get("request"))
            obj.update({ "watch": watch_id })
            result = Request(**obj)
            session.add(result)
            session.flush()
            session.commit()

        if msg.get("system"):
            obj = deepcopy(msg.get("system"))
            obj.update({ "watch": watch_id })
            systemResource.post(obj)

        if msg.get("lvgl"):
            obj = deepcopy(msg.get("lvgl"))
            obj.update({ "watch": watch_id })
            lvglResource.post(obj)

        if msg.get("evm"):
            obj = deepcopy(msg.get("evm"))
            obj.update({ "watch": watch_id })
            evmResource.post(obj)

        if msg.get("image"):
            obj = deepcopy(msg.get("image"))
            imageResource.post_array(obj, watch_id)

    def check_client(self, instance):
        index = -1
        for i, c in enumerate(instance._clients):
            if id(instance) == id(c.get("context")):
                index = i
                break
        return index

    def receive_message(self, instance, message):
        if message.get("type") and message.get("token"):
            # 获取token值，检验正确与否，获取uuid
            payload = jwt.decode(message.get("token"), config['SECRET_KEY'], verify=True, algorithms=['HS256'], options={ 'verify_aud': False })

            # 认证包，认证不通过，则剔除该连接
            if message.get("type") == "auth":
                if not message.get("token"):
                    return json.dumps({ "code": 400, "data": None, "msg": "token can not be null" })

                user = session.query(User).filter(User.id == payload.get("data").get("id")).one_or_none()
                if not user:
                    return json.dumps({ "code": 400, "data": None, "msg": "user not exists" })

                # 查询该用户可查看设备
                devices = session.query(Device).filter(Device.create_by==payload.get("data").get("id")).all()
                if len(devices):
                    instance._clients.append({
                        'uuid': payload.get("data").get("uuid"),
                        'context': instance,
                        'devices': list(map(lambda d:d.imei, devices)),
                        'ts': int(time.time())
                    })
                    return json.dumps({ 'code': 200, 'data': None, 'msg': 'auth passed' })
                else: # 没有设备，是否断开连接
                    return json.dumps({ 'code': 400, 'data': None, 'msg': 'no devices' })
            else:
                return json.dumps({ 'code': 200, 'data': None, 'msg': 'unkonw message packet, disconnect by server' })

    def broadcast_message(self, clients, message):
        if not message.get("imei"):
            return False

        for item in clients:
            if message.get("imei") in item.get("devices", []):
                item.get("context").write_message(json.dumps(message))

monitorManager = MonitorManager()